package de.codedo.java.editor;

import java.lang.reflect.Field;

public class ReflectionUtils
{
	public static Field getField(String fieldName, Object object) throws NoSuchFieldException
	{
		Field field = findField(fieldName, object.getClass());
		field.setAccessible(true);
		return field;
	}

	@SuppressWarnings("unchecked")
	public static <T> T getFieldValue(String fieldName, Object object) throws NoSuchFieldException, IllegalAccessException
	{
		Field field = getField(fieldName, object);
		return (T)field.get(object);
	}
	
	public static void setFieldValue(Object object, String fieldName, Object value) throws NoSuchFieldException, IllegalAccessException 
	{
		Field field = getField(fieldName, object);
		field.set(object, value);
	}
	
	private static Field findField(String fieldName, Class<?> clazz) throws NoSuchFieldException
	{
		try
		{
			return clazz.getDeclaredField(fieldName);
		}
		catch (NoSuchFieldException ex)
		{
			// OK, this class does not declare the field. Continue searching the superclass below.
		}

		Class<?> superClass = clazz.getSuperclass();
		if (superClass == null)
		{
			throw new IllegalStateException("Field " + fieldName + " could not be found in object's type hierarchy.");
		}
		return findField(fieldName, superClass);
	}
}
